@@ -3,7 +3,7 @@
 #' Function combines parameters of linear regressions of the first variable
 #' on all the other provided data using pAIC weights
 #'
-#' The algorithm uses lm() to fit different models and then combines the models
+#' The algorithm uses alm() to fit different models and then combines the models
 #' based on the selected point IC. This is a dynamic counterpart of
 #' \link[greybox]{lmCombine} function.
 #'
@@ -19,10 +19,32 @@
 #' one are produced and then combined.
 #' @param silent If \code{FALSE}, then nothing is silent, everything is printed
 #' out. \code{TRUE} means that nothing is produced.
+#' @param distribution Distribution to pass to \code{alm()}.
 #'
 #' @return Function returns \code{model} - the final model of the class
-#' "lm.combined", which includes time varying parameters and dynamic importance
-#' of each variable.
+#' "gryboxD", which includes time varying parameters and dynamic importance
+#' of each variable. The list of variables:
+#' \itemize{
+#' \item coefficients - mean (over time) parameters of the model,
+#' \item se - mean standard errors of the parameters of the model,
+#' \item actuals - actual values of the response variable,
+#' \item fitted.values - the fitted values,
+#' \item residuals - residual of the model,
+#' \item distribution - distribution used in the estimation,
+#' \item logLik - mean (over time) log-likelihood of the model,
+#' \item IC - dynamic values of the information criterion (pIC),
+#' \item df.residual - mean number of degrees of freedom of the residuals of
+#' the model,
+#' \item df - mean number of degrees of freedom of the model,
+#' \item importance - dynamic importance of the parameters,
+#' \item coefficientsDynamic - table with parameters of the model, varying over
+#' the time,
+#' \item df.residualDynamic - dynamic df.residual,
+#' \item dfDynamic - dynamic df,
+#' \item call - call used in the function,
+#' \item rank - rank of the combined model,
+#' \item model - the data used in the model.
+#' }
 #'
 #' @seealso \code{\link[greybox]{stepwise}, \link[greybox]{lmCombine}}
 #'
@@ -36,12 +58,17 @@
 #' outSample <- xreg[-c(1:80),]
 #' # Combine all the possible models
 #' ourModel <- lmDynamic(inSample,bruteForce=TRUE)
-#' forecast(ourModel,outSample)
-#' plot(forecast(ourModel,outSample))
+#' predict(ourModel,outSample)
+#' plot(predict(ourModel,outSample))
 #'
 #' @export lmDynamic
-lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, silent=TRUE){
+lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, silent=TRUE,
+                      distribution=c("dnorm","dfnorm","dlnorm","dlaplace","ds","dchisq","dlogis",
+                                    "plogis","pnorm")){
     # Function combines linear regression models and produces the combined lm object.
+    cl <- match.call();
+    cl$formula <- as.formula(paste0(colnames(data)[1]," ~ 1"));
+
     ourData <- data;
     if(!is.data.frame(ourData)){
         ourData <- as.data.frame(ourData);
@@ -66,6 +93,16 @@ lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, s
         IC <- pBICc;
     }
 
+    distribution <- distribution[1];
+    if(distribution=="dnorm"){
+        lmCall <- lm;
+        listToCall <- vector("list");
+    }
+    else{
+        lmCall <- alm;
+        listToCall <- list(distribution=distribution);
+    }
+
     # Observations in sample, assuming that the missing values are for the holdout
     obsInsample <- sum(!is.na(ourData[,1]));
     # Number of variables
@@ -97,7 +134,10 @@ lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, s
         parametersSE <- matrix(0,nCombinations,nVariables+1);
 
         # Starting estimating the models with just a constant
-        ourModel <- lm(as.formula(paste0(responseName,"~1")),data=ourData);
+        # ourModel <- alm(as.formula(paste0(responseName,"~1")),data=ourData,distribution=distribution);
+        listToCall$formula <- as.formula(paste0(responseName,"~1"));
+        listToCall$data <- ourData;
+        ourModel <- do.call(lmCall,listToCall);
         pICs[,1] <- pAIC(ourModel);
         parameters[1,1] <- coef(ourModel)[1];
         parametersSE[1,1] <- diag(vcov(ourModel));
@@ -107,21 +147,23 @@ lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, s
             if(!silent){
                 cat("Selecting the best model...\n");
             }
-            bestModel <- stepwise(ourData, ic=ic, method="kendall");
+            bestModel <- stepwise(ourData, ic=ic, distribution=distribution);
         }
         else{
-            bestModel <- stepwise(ourData, ic=ic);
+            bestModel <- stepwise(ourData, ic=ic, distribution=distribution);
         }
 
         # If the number of variables is small, do bruteForce
-        if(ncol(bestModel$model)<16){
+        if(nParam(bestModel)<16){
             newData <-  ourData[,c(colnames(ourData)[1],names(bestModel$ICs)[-1])];
-            return(lmDynamic(newData, ic=ic, bruteForce=TRUE, silent=silent));
+            bestModel <- lmDynamic(newData, ic=ic, bruteForce=TRUE, silent=silent, distribution=distribution);
+            bestModel$call <- cl;
+            return(bestModel);
         }
         # If we have too many variables, use "stress" analysis
         else{
             # Extract names of the used variables
-            bestExoNames <- colnames(bestModel$model)[-1];
+            bestExoNames <- names(coef(bestModel))[-1];
             nVariablesInModel <- length(bestExoNames);
             # Define number of combinations:
             # 1. Best model
@@ -180,7 +222,10 @@ lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, s
             cat(paste0(round(i/nCombinations,2)*100,"%"));
         }
         lmFormula <- paste0(responseName,"~",paste0(variablesNames[variablesCombinations[i,]==1],collapse="+"));
-        ourModel <- lm(as.formula(lmFormula),data=ourData);
+        # ourModel <- alm(as.formula(lmFormula),data=ourData,distribution=distribution);
+        listToCall$formula <- as.formula(lmFormula);
+        listToCall$data <- ourData;
+        ourModel <- do.call(lmCall,listToCall);
         pICs[,i] <- pAIC(ourModel);
         parameters[i,c(1,variablesCombinations[i,])==1] <- coef(ourModel);
         parametersSE[i,c(1,variablesCombinations[i,])==1] <- diag(vcov(ourModel));
@@ -220,21 +265,14 @@ lmDynamic <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteForce=FALSE, s
     parametersSECombined <- pICWeights %*% sqrt(parametersSE +(parameters - matrix(parametersMean,nrow(parameters),ncol(parameters),byrow=T))^2);
     colnames(parametersSECombined) <- exoNames;
 
-    # Create an object of the same name as the original data
-    # If it was a call on its own, make it one string
-    assign(paste0(deparse(substitute(data)),collapse=""),as.data.frame(data));
-    testModel <- do.call("lm", list(formula=as.formula(paste0(responseName,"~.")), data=substitute(data)));
-
     #Calcualte logLik
     logLikCombined <- sum(dnorm(errors,0,sd=sqrt(sum(errors^2)/df),log=TRUE));
 
-    ourTerms <- testModel$terms;
+    finalModel <- list(coefficients=parametersMean, se=parametersSECombined, fitted.values=as.vector(yFitted),
+                       residuals=as.vector(errors), distribution=distribution, logLik=logLikCombined, IC=ICValue,
+                       df.residual=mean(df), df=sum(apply(importance,2,mean))+1, importance=importance,
+                       coefficientsDynamic=parametersWeighted, df.residualDynamic=df, dfDynamic=apply(importance,1,sum)+1,
+                       call=cl, rank=nVariables+1, data=ourData);
 
-    finalModel <- list(coefficients=parametersMean, residuals=as.vector(errors), fitted.values=as.vector(yFitted),
-                       df.residual=mean(df), se=parametersSECombined, dynamic=parametersWeighted,
-                       importance=importance, IC=ICValue, call=testModel$call, logLik=logLikCombined, rank=nVariables+1,
-                       model=ourData, terms=ourTerms, qr=qr(ourData), df=sum(apply(importance,2,mean))+1,
-                       df.residualDynamic=df,dfDynamic=apply(importance,1,sum)+1);
-
-    return(structure(finalModel,class=c("greyboxD","greybox","lm")));
+    return(structure(finalModel,class=c("greyboxD","greybox","alm")));
 }
